% Creates random guide planks using level sets of a smooth function with
% up to two Gaussian bumps.
%
% Version 0.2, May 2017
% Author jlezama@fing.edu.uy

clear
%close all

seed = 13;
rng(seed)

% Options
PLOT = 1; % plot results
SAVE = 1; % save results
H = 4445; % height
W = 556; % width

% Create results folder structure
if SAVE
    outfolder = 'synthetic_examples'
    system(sprintf('mkdir -p %s', outfolder))
    system(sprintf('mkdir -p %s/params', outfolder))
end

for iteration = 1;
    %close all
    
    % Set parameters randomly for two Gaussians
    sigma_blob = [3000 2000];%random_choose([200, 400, 800, 1200], 2); % 300
    sigma_displacement = random_choose([25, 75, 50, 100]); % 60
    max_displacement = random_choose([ 4, 6, 8 ,12]);
    blob_vertical_position =  [2/8 31/32];%random_choose([0 1/4 1/3 1/2 2/3 3/4 1], 2) ;
    blob_horizontal_position = [1/4 1/2];%random_choose([0 1/4 1/3 1/2 2/3 3/4 1], 2);
    num_intervals = 20;%random_choose([15, 20, 30] );
    blob_multiplier = [1 1];%random_choose([0,0.5,  1, 2],2);
    dilation_length_vertical = 10;%random_choose([6, 10, 12]);
    dilation_length_horizontal = 2;%random_choose([6, 10, 12]);
    surface_decay = 1.5;%random_choose([1, 1.5, 2, 3]);
    vertical_displacement_multiplier = random_choose([8, 10, 12]);
    
    outputfname = random_string(20);
    
    % Initalize surface
    [X, Y] = meshgrid(1:W,1:H);
    initial_surface = (1-normalize(abs(X-W/2).^surface_decay));
    
    % Add two gaussian bumps
    disp('generating gaussian blob 1')
    g1 = gaussian_blob(H,W, round(H*blob_vertical_position(1)), round(W*blob_horizontal_position(1)), [30 0;0 7]*sigma_blob(1));
    
    disp('generating gaussian blob 2')
    g2 = gaussian_blob(H,W, round(H*blob_vertical_position(2)), round(W*blob_horizontal_position(2)), [2 0;0 70]*sigma_blob(2));
    
    mysurface = initial_surface + g1*blob_multiplier(1) + g2*blob_multiplier(2);
    mysurface = normalize(mysurface);
    
    % Add some random displacement
    disp('generating random displacement')
    [vx, vy] = random_displacement(H, W, sigma_displacement, max_displacement, 0, 0 ,0);
    
    [X, Y] = meshgrid(1:W,1:H);
    mysurface = interp2(mysurface, X + vx , Y + vy*vertical_displacement_multiplier); % stronger on vertical
    mysurface = normalize(mysurface);
    mysurface(isnan(mysurface))=0;
      
    if PLOT
        figure, surf(mysurface*100, 'edgecolor','none'), axis equal
    end
    
    % Compute Level Lines
    intervals = [0:1./num_intervals:1];
    intervals = intervals.^2; % stretch intervals to have larger gaps in cathedrals
    intervals = sort(normalize(intervals + randn(size(intervals))/300));
    L = length(intervals);
    
    % Colorize
    red = zeros(H,W);
    green = zeros(H,W);
    blue = zeros(H,W);
    
    light = [179, 150, 108
        158, 133, 93
        186, 158, 110
        167, 136, 92
        175, 150, 109
        211, 181, 131]/255;
    
    dark = [110, 78, 29
        114, 81, 36
        112, 82, 32
        107, 75, 18
        132, 96, 46
        139, 110, 66]/255;
    
    assert(size(dark,1)==size(light,1))
    Ncolors = size(dark,1);
    
    disp('painting intervals')
    for i=1:L-1
        z = find(mysurface>=intervals(i) & mysurface<=intervals(i+1));
        
        if mod(i,2)
            colors = dark;
        else
            colors = light;
        end
        
        c = randi(Ncolors);
        red(z) = colors(c,1);
        green(z) = colors(c,2);
        blue(z) = colors(c,3);
    end
    
    new_colored = cat(3,red,green,blue);
    
    % Cut borders
    new_colored = new_colored(H/20+1:end-H/20,W/20+1:end-W/20,:);
    
    disp('postprocessing gradients')
    new_postprocessed = gradient_to_guide(normalize(imgradient(rgb2gray(new_colored))), dilation_length_vertical, dilation_length_horizontal, dark, light);
    %new_postprocessed = new_colored;
    
    % Plot
    if PLOT
        figure,
        subplot(1,3,1)
        contourf(flipud(mysurface), intervals)
        colormap gray
        axis image
        title('level lines')
        subplot(1,3,2)
        imshow(new_colored)
        title('level sets filled')
        subplot(1,3,3)
        imshow(new_postprocessed)
        title('level lines postprocessed')
    end
    
    % Save results
    if SAVE
        imwrite(new_colored, sprintf('%s/%s.png', outfolder, outputfname));
        imwrite(new_postprocessed, sprintf('%s/%s_postprocessed.png', outfolder, outputfname));
        disp('saving parameters')
        system(sprintf('cp %s.m %s/params/%s.m', mfilename(), outfolder, outputfname))
        save(sprintf('%s/params/%s.mat', outfolder, outputfname), ...
            'sigma_blob', 'sigma_displacement', 'max_displacement', ...
            'blob_vertical_position', 'blob_horizontal_position', 'num_intervals', ...
            'blob_multiplier', 'dilation_length_vertical', 'surface_decay');
    end
    
    % show only final plank
    a(:,:,1) = new_postprocessed(:,:,1)'; a(:,:,2) = new_postprocessed(:,:,2)'; a(:,:,3) = new_postprocessed(:,:,3)';figure; imshow(a)
end