function [vx, vy] = random_displacement(H, W, sigma, max_displacement, modulate, sigma2, DELTA);
% [vx, vy] = RANDOM_DISPLACEMENT(H, W, sigma, max_displacement, modulate, sigma2, DELTA)
%
% Generates a random 2D displacement vectors field. If 'modulate' is set,
% it will modulate the vector field to have radial shape. A finer-scale 
% random field can be added using sigma2 and DELTA
%
% Inputs:
% H: height of the field
% W: width of the field
% sigma: sigma  for Gaussian smoothing of random field
% max_displacement: max length of displacement vectors
% modulate: true if impose radial modulation
% sigma2: sigma for Gaussian smoothing of random field to be added to
% radially-modulated field
% DELTA: ratio of the small additive random field w.r.t. output random
% field
%
% March 15 2017 
% Author jlezama@fing.edu.uy



if nargin <5
    modulate = 0;
end



gauss_filt = fspecial('gaussian', sigma*7, sigma);

%gauss_filt = gaussian_custom(sigma*7,[1 .3;1 .5]*sigma);

%
vx = imfilter(randn([H W]), gauss_filt);
vy = imfilter(randn([H W]), gauss_filt);

vx = vx/max(abs(vx(:))); % limit to -1,1
vy = vy/max(abs(vy(:)));
% 


if modulate
    % create mask to force vx and vy into radial shape
    [X, Y] = meshgrid(1:W, 1:H);
    a=atan2(X-H/2,Y-W/2);
    vx_modulator = abs(sin(a));
    vy_modulator = abs(cos(a));
    
    
    % impose sign for radial
    W2 = round(W/2);
    H2 = round(H/2);
    vx(:,1:W2)=vx(:,1:W2).*sign(vx(:,1:W2))*-1;
    vx(:,W2+1:end)=vx(:,W2+1:end).*sign(vx(:,W2+1:end));
    
    vy(1:H2,:)=vy(1:H2,:).*sign(vy(1:H2,:))*-1;
    vy(H2+1:end,:)=vy(H2+1:end,:).*sign(vy(H2+1:end,:));
    
    % apply modulation
    vx = vx.*vx_modulator;
    vy = vy.*vy_modulator;
    
    
    % add a small non-radial component
    [vx_small, vy_small] = random_displacement(H, W, sigma2, 1,0);
    
    vx = vx+vx_small/DELTA;
    vy = vy+vy_small/DELTA;
end


vx = vx*max_displacement;
vy = vy*max_displacement;